import _toConsumableArray from 'babel-runtime/helpers/toConsumableArray';
import _typeof from 'babel-runtime/helpers/typeof';
import _extends from 'babel-runtime/helpers/extends';
import _classCallCheck from 'babel-runtime/helpers/classCallCheck';
import _createClass from 'babel-runtime/helpers/createClass';
import _possibleConstructorReturn from 'babel-runtime/helpers/possibleConstructorReturn';
import _inherits from 'babel-runtime/helpers/inherits';
import React from 'react';
import html from 'react-inner-html';
import stringifyObject from 'stringify-object';
import { sourceStyles } from '../utils/sourceStyles';
import { types } from '../utils/types';
import { randstr, cleanArray, filterNode } from '../utils/helpers';

var Terminal = function (_React$Component) {
  _inherits(Terminal, _React$Component);

  function Terminal(props) {
    _classCallCheck(this, Terminal);

    var _this = _possibleConstructorReturn(this, (Terminal.__proto__ || Object.getPrototypeOf(Terminal)).call(this, props));

    _this.state = {
      rootTracker: randstr(),
      inputTracker: randstr(),
      commands: {},
      stdout: [],
      history: [],
      historyPosition: null
    };

    _this._getTerminalNode = _this._getTerminalNode.bind(_this);
    _this._getRootNode = _this._getRootNode.bind(_this);
    _this.focusTerminal = _this.focusTerminal.bind(_this);
    _this.validateCommands = _this.validateCommands.bind(_this);
    _this.showWelcomeMessage = _this.showWelcomeMessage.bind(_this);
    _this.showHelp = _this.showHelp.bind(_this);
    _this.pushToStdout = _this.pushToStdout.bind(_this);
    _this.getStdout = _this.getStdout.bind(_this);
    _this.clearStdout = _this.clearStdout.bind(_this);
    _this.processCommand = _this.processCommand.bind(_this);
    _this.handleInput = _this.handleInput.bind(_this);
    return _this;
  }

  _createClass(Terminal, [{
    key: '_getTerminalNode',
    value: function _getTerminalNode() {
      var _this2 = this;

      var elements = document.getElementsByName('react-console-emulator__input');
      return filterNode(elements, function (el) {
        return el.dataset.input === _this2.state.inputTracker;
      });
    }
  }, {
    key: '_getRootNode',
    value: function _getRootNode() {
      var _this3 = this;

      var elements = document.getElementsByName('react-console-emulator');
      return filterNode(elements, function (el) {
        return el.dataset.terminal === _this3.state.rootTracker;
      });
    }

    /**
     * Manually push to the stdout of a terminal. Use with caution.
     * @param {String} message The message to output to the terminal. If not using safe mode, make sure to XSS-proof this.
     * @param {Boolean} dangerMode If true, set output content with innerHTML. Dangerous.
     * @param {HTMLElement} contentElement The content element of the terminal you want to push output to. Uses first found element if omitted.
     * @param {HTMLElement} inputElement The input element of the terminal you want to push output to. Uses first found element if omitted.
     * @param {HTMLElement} inputAreaElement The input area element of the terminal you want to push output to. Uses first found element if omitted.
     */

  }, {
    key: 'focusTerminal',
    value: function focusTerminal() {
      this._getTerminalNode().focus();
    }
  }, {
    key: 'scrollToBottom',
    value: function scrollToBottom() {
      var rootNode = this._getRootNode();

      // This may look ridiculous, but it is required to decouple execution for just a millisecond in order to scroll all the way
      setTimeout(function () {
        rootNode.scrollTop = rootNode.scrollHeight;
      }, 1);
    }
  }, {
    key: 'validateCommands',
    value: function validateCommands() {
      var immutables = ['help', 'clear'];
      var commands = this.props.commands;

      var defaultCommands = {
        help: {
          description: 'Show a list of available commands.',
          fn: this.showHelp
        },
        clear: {
          description: 'Empty the terminal window.',
          explicitExec: true,
          fn: this.clearStdout
        }
      };

      var validCommands = void 0;

      // Pre-register defaults
      if (!this.props.noDefaults) validCommands = _extends({}, defaultCommands);else validCommands = {};

      for (var c in commands) {
        // Check that command contains a function
        if (typeof commands[c].fn !== 'function') throw new Error('\'fn\' property of command \'' + c + '\' is invalid; expected \'function\', got \'' + _typeof(commands[c].fn) + '\'');

        // Check that the command does not attempt to override immutables
        if (!this.props.noDefaults && immutables.includes(c)) throw new Error('Attempting to overwrite default command \'' + immutables[immutables.indexOf(c)] + '\'; cannot override default commands');

        // Add description if missing
        if (!commands[c].description) commands[c].description = 'None';
        validCommands[c] = commands[c];
      }

      this.setState({ commands: validCommands });
    }
  }, {
    key: 'showWelcomeMessage',
    value: function showWelcomeMessage() {
      var _this4 = this;

      var msg = this.props.welcomeMessage;

      if (typeof msg === 'boolean') this.pushToStdout('Welcome to the React terminal! Type \'help\' to get a list of commands.');else if (Array.isArray(msg)) msg.map(function (item) {
        return _this4.pushToStdout(item);
      });else this.pushToStdout(msg);
    }
  }, {
    key: 'showHelp',
    value: function showHelp() {
      for (var c in this.state.commands) {
        var cmdObj = this.state.commands[c];
        var usage = cmdObj.usage ? ' - ' + cmdObj.usage : '';
        this.pushToStdout(c + ' - ' + cmdObj.description + usage);
      }
    }
  }, {
    key: 'pushToStdout',
    value: function pushToStdout(message, rawInput) {
      var stdout = this.state.stdout;
      var history = this.state.history;

      stdout.push(message);

      if (rawInput) {
        history.push(rawInput);
        this.setState({ stdout: stdout, history: history, historyPosition: null });
      } else this.setState({ stdout: stdout });
    }
  }, {
    key: 'getStdout',
    value: function getStdout() {
      var _this5 = this;

      var lineStyle = {
        margin: '0',
        lineHeight: '21px'
      };

      return this.state.stdout.map(function (line, i) {
        var safe = React.createElement(
          'p',
          { key: i, style: lineStyle },
          line
        );
        var dangerous = React.createElement('p', _extends({ key: i, style: lineStyle }, html(line)));

        return _this5.props.dangerMode ? dangerous : safe;
      });
    }
  }, {
    key: 'clearStdout',
    value: function clearStdout() {
      this.setState({ stdout: [] });
    }
  }, {
    key: 'clearInput',
    value: function clearInput() {
      this.setState({ historyPosition: null });
      this._getTerminalNode().value = '';
    }
  }, {
    key: 'processCommand',
    value: function processCommand() {
      var commandResult = { command: null, args: [], rawInput: null, result: null };

      var rawInput = this._getTerminalNode().value;

      var input = rawInput.split(' ');
      var command = input.splice(0, 1)[0]; // Removed portion is returned...
      var args = input; // ...and the rest can be used

      if (!this.props.noAutomaticStdout) {
        if (!this.props.noHistory) this.pushToStdout((this.props.promptLabel || '$') + ' ' + rawInput, rawInput);else this.pushToStdout((this.props.promptLabel || '$') + ' ' + rawInput);
      }

      if (rawInput) {
        commandResult.rawInput = rawInput;
        commandResult.command = command;
        commandResult.args = args;

        var cmdObj = this.state.commands[command];

        if (!cmdObj) this.pushToStdout(this.props.errorText ? this.props.errorText.replace(/\[command\]/gi, command) : 'Command \'' + command + '\' not found!');else {
          var res = cmdObj.fn.apply(cmdObj, _toConsumableArray(args));

          this.pushToStdout(res);
          commandResult.result = res;
          if (cmdObj.explicitExec) cmdObj.fn.apply(cmdObj, _toConsumableArray(args));
        }
      }

      this.clearInput();
      if (!this.props.noAutoScroll) this.scrollToBottom();
      if (this.props.commandCallback) this.props.commandCallback(commandResult);
    }
  }, {
    key: 'scrollHistory',
    value: function scrollHistory(direction) {
      var history = cleanArray(this.state.history).reverse(); // Clean empty items and reverse order to ease position tracking
      var position = this.state.historyPosition;
      var termNode = this._getTerminalNode();

      if (!this.state.noAutomaticStdout && history.length > 0) {
        // Only run if history is non-empty and in use
        switch (direction) {
          case 'up':
            if (position === null) {
              // If not touched, get most recent
              termNode.value = history[0];
              this.setState({ historyPosition: 0 });
            } else if (position + 1 === history.length) {
              // If the last item will be reached on this press, get first entry and decrement position by 1 to avoid confusing downscroll
              termNode.value = history[history.length - 1];
              this.setState({ historyPosition: history.length - 1 });
            } else {
              var atBottom = position - 1 === -1; // -1 for zero-based index

              // If at last item, increment by one more to avoid showing the same item twice
              termNode.value = atBottom ? history[position + 1] : history[position];
              this.setState({ historyPosition: atBottom ? position + 2 : position + 1 });
            }
            break;
          case 'down':
            if (position === null || !history[position]) {
              // If at initial or out of range, clear (Unix-like behaviour)
              termNode.value = '';
              this.setState({ historyPosition: null });
            } else if (position - 1 === -1) {
              // If position will go negative on this press, bottom has been reached - reset
              termNode.value = history[0];
              this.setState({ historyPosition: null });
            } else {
              var reachedFirst = position + 1 === history.length; // +1 for zero-based index

              // If at first item, decrement by one more to avoid showing the same item twice
              termNode.value = reachedFirst ? history[position - 1] : history[position];
              this.setState({ historyPosition: reachedFirst ? position - 2 : position - 1 });
            }
            break;
        }
      }
    }
  }, {
    key: 'handleInput',
    value: function handleInput(event) {
      switch (event.key) {
        case 'Enter':
          this.processCommand();break;
        case 'ArrowUp':
          this.scrollHistory('up');break;
        case 'ArrowDown':
          this.scrollHistory('down');break;
      }
    }
  }, {
    key: 'componentDidUpdate',
    value: function componentDidUpdate(prevProps) {
      var oldCommands = stringifyObject(prevProps.commands);
      var currentCommands = stringifyObject(this.props.commands);

      // There was a change in commands
      if (oldCommands !== currentCommands) {
        // Re-validate
        this.validateCommands();
      }
    }
  }, {
    key: 'componentDidMount',
    value: function componentDidMount() {
      this.validateCommands();
      if (this.props.welcomeMessage) this.showWelcomeMessage();
      if (this.props.autoFocus) this.focusTerminal();
    }
  }, {
    key: 'render',
    value: function render() {
      var styles = {
        container: _extends({}, sourceStyles.container, {
          background: this.props.background || '#212121',
          backgroundSize: this.props.backgroundSize || 'cover'
        }),
        content: _extends({}, sourceStyles.content, {
          color: this.props.textColor || '#FFFFFF'
        }),
        inputArea: _extends({}, sourceStyles.inputArea),
        prompt: _extends({}, sourceStyles.prompt, {
          color: this.props.promptLabelColor || '#EE9C34'
        }),
        input: _extends({}, sourceStyles.input, {
          color: this.props.promptTextColor || '#F0BF81'
        })
      };

      return React.createElement(
        'div',
        {
          className: this.props.className || null,
          'data-terminal': this.state.rootTracker,
          name: 'react-console-emulator',
          style: styles.container,
          onClick: this.focusTerminal
        },
        React.createElement(
          'div',
          {
            className: this.props.contentClassName || null,
            name: 'react-console-emulator__content',
            style: styles.content
          },
          this.getStdout(),
          React.createElement(
            'div',
            {
              className: this.props.inputAreaClassName || null,
              name: 'react-console-emulator__inputArea',
              style: styles.inputArea
            },
            React.createElement(
              'span',
              {
                className: this.props.promptLabelClassName || null,
                style: styles.prompt
              },
              this.props.promptLabel || '$'
            ),
            React.createElement('input', {
              className: this.props.inputClassName || null,
              name: 'react-console-emulator__input',
              'data-input': this.state.inputTracker,
              style: styles.input,
              type: 'text',
              onKeyDown: this.handleInput
            })
          )
        )
      );
    }
  }], [{
    key: 'manualPushToStdout',
    value: function manualPushToStdout(message, dangerMode, contentElement, inputElement, inputAreaElement) {
      var content = contentElement || document.getElementsByName('react-console-emulator__content')[0];
      var input = inputElement || document.getElementsByName('react-console-emulator__input')[0];
      var inputArea = inputAreaElement || document.getElementsByName('react-console-emulator__inputArea')[0];

      var messageElement = document.createElement('p');

      if (dangerMode) messageElement.innerHTML = message;else messageElement.innerText = message;

      messageElement.style = 'margin: 0px; line-height: 21px;';

      content.appendChild(messageElement);
      content.appendChild(inputArea);
      input.value = '';
      input.focus();
    }
  }]);

  return Terminal;
}(React.Component);

Terminal.propTypes = _extends({}, types);
export default Terminal;